/*
 * Copyright 2014-2015 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.generator;

import org.junit.Assert;
import org.junit.Test;

import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.testbase.TestBaseQuery;
import ru.ispras.testbase.TestBaseQueryBuilder;
import ru.ispras.testbase.TestData;
import ru.ispras.testbase.knowledge.iterator.Iterator;

/**
 * {@link GeneratorRangeTestCase} creates and executes a TestBase query using the
 * GeneratorRange generator, which results in iterator a over a collection of test data.
 * This test is needed to see whether the combination logic works as expected.
 *
 * @author <a href="mailto:andrewt@ispras.ru">Andrei Tatarnikov</a>
 */
public final class GeneratorRangeTestCase {
  private static final int MIN = 1;
  private static final int MAX = 5;

  private static int RANGE_LENGTH = MAX - MIN + 1;
  private static int VARIABLE_COUNT = 5;

  private static int DATA_SET_COUNT = power(RANGE_LENGTH, VARIABLE_COUNT);

  private static TestBaseQuery newTestBaseQuery() {
    final TestBaseQueryBuilder builder = new TestBaseQueryBuilder();

    builder.setParameter("min", MIN);
    builder.setParameter("max", MAX);

    for (int index = 0; index < VARIABLE_COUNT; ++index) {
      final String variableName = String.format("var%d", index + 1);
      builder.setBinding(variableName, NodeVariable.newBitVector(variableName, 8));
    }

    return builder.build();
  }

  @Test
  public void test() {
    final DataGenerator dataGenerator = new GeneratorRange();
    final Iterator<TestData> iterator = dataGenerator.generate(newTestBaseQuery());

    int count = 0;
    for (iterator.init(); iterator.hasValue(); iterator.next()) {
      //System.out.println(iterator.value().toString());
      count++;
    }

    // To detect changes in the implementation.
    Assert.assertEquals(DATA_SET_COUNT, count);
  }

  private static int power(final int value, final int power) {
    return 0 == power ? 1 : value * power(value, power - 1);
  }
}
